@@ -148,14 +148,10 @@ int		 get_addrs(const char *, struct server *, in_port_t);
 int		 addr_dup_check(struct addresslist *, struct address *,
 		    const char *, const char *);
 int		 add_addr(struct server *, struct address *);
-struct address	*host_v4(const char *);
-struct address	*host_v6(const char *);
-int		 host_dns(const char *, struct server *,
+int		 host(const char *, struct server *,
 		    int, in_port_t, const char *, int);
 int		 host_if(const char *, struct server *,
 		    int, in_port_t, const char *, int);
-int		 host(const char *, struct server *,
-		    int, in_port_t, const char *, int);
 int		 is_if_in_group(const char *, const char *);
 
 typedef struct {
@@ -168,7 +164,7 @@ typedef struct {
 } YYSTYPE;
 
 
-#line 172 "parse.c"
+#line 168 "parse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -726,11 +722,11 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   137,   137,   138,   139,   140,   141,   142,   145,   163,
-     179,   180,   183,   190,   204,   207,   217,   220,   236,   251,
-     251,   269,   279,   289,   299,   309,   318,   328,   338,   345,
-     368,   372,   375,   378,   381,   384,   387,   390,   393,   399,
-     400,   403,   406,   407
+       0,   133,   133,   134,   135,   136,   137,   138,   141,   159,
+     173,   174,   183,   190,   204,   212,   229,   232,   247,   262,
+     262,   280,   290,   300,   310,   320,   329,   339,   349,   356,
+     377,   384,   387,   390,   393,   396,   399,   402,   410,   420,
+     421,   424,   427,   428
 };
 #endif
 
@@ -1344,13 +1340,13 @@ yyreduce:
   switch (yyn)
     {
   case 7: /* grammar: grammar error '\n'  */
-#line 142 "parse.y"
+#line 138 "parse.y"
                                                 { file->errors++; }
-#line 1350 "parse.c"
+#line 1346 "parse.c"
     break;
 
   case 8: /* varset: STRING '=' STRING  */
-#line 145 "parse.y"
+#line 141 "parse.y"
                                         {
 			char *s = (yyvsp[-2].v.string);
 			while (*s++) {
@@ -1367,19 +1363,17 @@ yyreduce:
 			free((yyvsp[-2].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1371 "parse.c"
+#line 1367 "parse.c"
     break;
 
   case 9: /* boolean: STRING  */
-#line 163 "parse.y"
+#line 159 "parse.y"
                          {
 			if (strcasecmp((yyvsp[0].v.string), "1") == 0 ||
-			    strcasecmp((yyvsp[0].v.string), "yes") == 0 ||
 			    strcasecmp((yyvsp[0].v.string), "on") == 0)
 				(yyval.v.number) = 1;
 			else if (strcasecmp((yyvsp[0].v.string), "0") == 0 ||
-			    strcasecmp((yyvsp[0].v.string), "off") == 0 ||
-			    strcasecmp((yyvsp[0].v.string), "no") == 0)
+			    strcasecmp((yyvsp[0].v.string), "off") == 0)
 				(yyval.v.number) = 0;
 			else {
 				yyerror("invalid boolean value '%s'", (yyvsp[0].v.string));
@@ -1388,18 +1382,24 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1392 "parse.c"
+#line 1386 "parse.c"
     break;
 
   case 10: /* boolean: ON  */
-#line 179 "parse.y"
+#line 173 "parse.y"
                      { (yyval.v.number) = 1; }
-#line 1398 "parse.c"
+#line 1392 "parse.c"
     break;
 
   case 11: /* boolean: NUMBER  */
-#line 180 "parse.y"
-                         { (yyval.v.number) = (yyvsp[0].v.number); }
+#line 174 "parse.y"
+                         {
+			if ((yyvsp[0].v.number) != 0 && (yyvsp[0].v.number) != 1) {
+				yyerror("invalid boolean value '%lld'", (yyvsp[0].v.number));
+				YYERROR;
+			}
+			(yyval.v.number) = (yyvsp[0].v.number);
+		}
 #line 1404 "parse.c"
     break;
 
@@ -1435,14 +1435,26 @@ yyreduce:
   case 14: /* main: PREFORK NUMBER  */
 #line 204 "parse.y"
                                  {
+			if ((yyvsp[0].v.number) <= 0 || (yyvsp[0].v.number) > PROC_MAX_INSTANCES) {
+				yyerror("prefork is %s: %lld",
+				    (yyvsp[0].v.number) <= 0 ? "too small" : "too large", (yyvsp[0].v.number));
+				YYERROR;
+			}
 			gotwebd->prefork_gotwebd = (yyvsp[0].v.number);
 		}
-#line 1441 "parse.c"
+#line 1446 "parse.c"
     break;
 
   case 15: /* main: CHROOT STRING  */
-#line 207 "parse.y"
+#line 212 "parse.y"
                                 {
+			if (*(yyvsp[0].v.string) == '\0') {
+				yyerror("chroot path can't be an empty"
+				    " string");
+				free((yyvsp[0].v.string));
+				YYERROR;
+			}
+
 			n = strlcpy(gotwebd->httpd_chroot, (yyvsp[0].v.string),
 			    sizeof(gotwebd->httpd_chroot));
 			if (n >= sizeof(gotwebd->httpd_chroot)) {
@@ -1452,24 +1464,23 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1456 "parse.c"
+#line 1468 "parse.c"
     break;
 
   case 16: /* main: UNIX_SOCKET boolean  */
-#line 217 "parse.y"
+#line 229 "parse.y"
                                       {
 			gotwebd->unix_socket = (yyvsp[0].v.number);
 		}
-#line 1464 "parse.c"
+#line 1476 "parse.c"
     break;
 
   case 17: /* main: UNIX_SOCKET_NAME STRING  */
-#line 220 "parse.y"
+#line 232 "parse.y"
                                           {
 			n = snprintf(gotwebd->unix_socket_name,
 			    sizeof(gotwebd->unix_socket_name), "%s%s",
-			    strlen(gotwebd->httpd_chroot) ?
-			    gotwebd->httpd_chroot : D_HTTPD_CHROOT, (yyvsp[0].v.string));
+			    gotwebd->httpd_chroot, (yyvsp[0].v.string));
 			if (n < 0 ||
 			    (size_t)n >= sizeof(gotwebd->unix_socket_name)) {
 				yyerror("%s: unix_socket_name truncated",
@@ -1479,11 +1490,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1483 "parse.c"
+#line 1494 "parse.c"
     break;
 
   case 18: /* server: SERVER STRING  */
-#line 236 "parse.y"
+#line 247 "parse.y"
                                 {
 			struct server *srv;
 
@@ -1499,11 +1510,11 @@ yyreduce:
 			log_debug("adding server %s", (yyvsp[0].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1503 "parse.c"
+#line 1514 "parse.c"
     break;
 
   case 19: /* $@1: %empty  */
-#line 251 "parse.y"
+#line 262 "parse.y"
                                 {
 			struct server *srv;
 
@@ -1519,18 +1530,18 @@ yyreduce:
 			log_debug("adding server %s", (yyvsp[0].v.string));
 			free((yyvsp[0].v.string));
 		}
-#line 1523 "parse.c"
+#line 1534 "parse.c"
     break;
 
   case 20: /* server: SERVER STRING $@1 '{' optnl serveropts2 '}'  */
-#line 265 "parse.y"
+#line 276 "parse.y"
                                             {
 		}
-#line 1530 "parse.c"
+#line 1541 "parse.c"
     break;
 
   case 21: /* serveropts1: REPOS_PATH STRING  */
-#line 269 "parse.y"
+#line 280 "parse.y"
                                     {
 			n = strlcpy(new_srv->repos_path, (yyvsp[0].v.string),
 			    sizeof(new_srv->repos_path));
@@ -1541,11 +1552,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1545 "parse.c"
+#line 1556 "parse.c"
     break;
 
   case 22: /* serveropts1: SITE_NAME STRING  */
-#line 279 "parse.y"
+#line 290 "parse.y"
                                    {
 			n = strlcpy(new_srv->site_name, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_name));
@@ -1556,11 +1567,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1560 "parse.c"
+#line 1571 "parse.c"
     break;
 
   case 23: /* serveropts1: SITE_OWNER STRING  */
-#line 289 "parse.y"
+#line 300 "parse.y"
                                     {
 			n = strlcpy(new_srv->site_owner, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_owner));
@@ -1571,11 +1582,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1575 "parse.c"
+#line 1586 "parse.c"
     break;
 
   case 24: /* serveropts1: SITE_LINK STRING  */
-#line 299 "parse.y"
+#line 310 "parse.y"
                                    {
 			n = strlcpy(new_srv->site_link, (yyvsp[0].v.string),
 			    sizeof(new_srv->site_link));
@@ -1586,11 +1597,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1590 "parse.c"
+#line 1601 "parse.c"
     break;
 
   case 25: /* serveropts1: LOGO STRING  */
-#line 309 "parse.y"
+#line 320 "parse.y"
                               {
 			n = strlcpy(new_srv->logo, (yyvsp[0].v.string), sizeof(new_srv->logo));
 			if (n >= sizeof(new_srv->logo)) {
@@ -1600,11 +1611,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1604 "parse.c"
+#line 1615 "parse.c"
     break;
 
   case 26: /* serveropts1: LOGO_URL STRING  */
-#line 318 "parse.y"
+#line 329 "parse.y"
                                   {
 			n = strlcpy(new_srv->logo_url, (yyvsp[0].v.string),
 			    sizeof(new_srv->logo_url));
@@ -1615,11 +1626,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1619 "parse.c"
+#line 1630 "parse.c"
     break;
 
   case 27: /* serveropts1: CUSTOM_CSS STRING  */
-#line 328 "parse.y"
+#line 339 "parse.y"
                                     {
 			n = strlcpy(new_srv->custom_css, (yyvsp[0].v.string),
 			    sizeof(new_srv->custom_css));
@@ -1630,11 +1641,11 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1634 "parse.c"
+#line 1645 "parse.c"
     break;
 
   case 28: /* serveropts1: LISTEN ON STRING fcgiport  */
-#line 338 "parse.y"
+#line 349 "parse.y"
                                             {
 			if (get_addrs((yyvsp[-1].v.string), new_srv, (yyvsp[0].v.port)) == -1) {
 				yyerror("could not get addrs");
@@ -1642,14 +1653,13 @@ yyreduce:
 			}
 			new_srv->fcgi_socket = 1;
 		}
-#line 1646 "parse.c"
+#line 1657 "parse.c"
     break;
 
   case 29: /* serveropts1: LISTEN ON SOCKET STRING  */
-#line 345 "parse.y"
+#line 356 "parse.y"
                                           {
-			if (!strcasecmp((yyvsp[0].v.string), "off") ||
-			    !strcasecmp((yyvsp[0].v.string), "no")) {
+			if (strcasecmp((yyvsp[0].v.string), "off") == 0) {
 				new_srv->unix_socket = 0;
 				free((yyvsp[0].v.string));
 				YYACCEPT;
@@ -1659,8 +1669,7 @@ yyreduce:
 
 			n = snprintf(new_srv->unix_socket_name,
 			    sizeof(new_srv->unix_socket_name), "%s%s",
-			    strlen(gotwebd->httpd_chroot) ?
-			    gotwebd->httpd_chroot : D_HTTPD_CHROOT, (yyvsp[0].v.string));
+			    gotwebd->httpd_chroot, (yyvsp[0].v.string));
 			if (n < 0 ||
 			    (size_t)n >= sizeof(new_srv->unix_socket_name)) {
 				yyerror("%s: unix_socket_name truncated",
@@ -1670,85 +1679,97 @@ yyreduce:
 			}
 			free((yyvsp[0].v.string));
 		}
-#line 1674 "parse.c"
-    break;
-
-  case 30: /* serveropts1: MAX_REPOS NUMBER  */
-#line 368 "parse.y"
-                                   {
-			if ((yyvsp[0].v.number) > 0)
-				new_srv->max_repos = (yyvsp[0].v.number);
-		}
 #line 1683 "parse.c"
     break;
 
+  case 30: /* serveropts1: MAX_REPOS NUMBER  */
+#line 377 "parse.y"
+                                   {
+			if ((yyvsp[0].v.number) <= 0) {
+				yyerror("max_repos is too small: %lld", (yyvsp[0].v.number));
+				YYERROR;
+			}
+			new_srv->max_repos = (yyvsp[0].v.number);
+		}
+#line 1695 "parse.c"
+    break;
+
   case 31: /* serveropts1: SHOW_SITE_OWNER boolean  */
-#line 372 "parse.y"
+#line 384 "parse.y"
                                           {
 			new_srv->show_site_owner = (yyvsp[0].v.number);
 		}
-#line 1691 "parse.c"
+#line 1703 "parse.c"
     break;
 
   case 32: /* serveropts1: SHOW_REPO_OWNER boolean  */
-#line 375 "parse.y"
+#line 387 "parse.y"
                                           {
 			new_srv->show_repo_owner = (yyvsp[0].v.number);
 		}
-#line 1699 "parse.c"
+#line 1711 "parse.c"
     break;
 
   case 33: /* serveropts1: SHOW_REPO_AGE boolean  */
-#line 378 "parse.y"
+#line 390 "parse.y"
                                         {
 			new_srv->show_repo_age = (yyvsp[0].v.number);
 		}
-#line 1707 "parse.c"
+#line 1719 "parse.c"
     break;
 
   case 34: /* serveropts1: SHOW_REPO_DESCRIPTION boolean  */
-#line 381 "parse.y"
+#line 393 "parse.y"
                                                 {
 			new_srv->show_repo_description = (yyvsp[0].v.number);
 		}
-#line 1715 "parse.c"
+#line 1727 "parse.c"
     break;
 
   case 35: /* serveropts1: SHOW_REPO_CLONEURL boolean  */
-#line 384 "parse.y"
+#line 396 "parse.y"
                                              {
 			new_srv->show_repo_cloneurl = (yyvsp[0].v.number);
 		}
-#line 1723 "parse.c"
+#line 1735 "parse.c"
     break;
 
   case 36: /* serveropts1: RESPECT_EXPORTOK boolean  */
-#line 387 "parse.y"
+#line 399 "parse.y"
                                            {
 			new_srv->respect_exportok = (yyvsp[0].v.number);
 		}
-#line 1731 "parse.c"
+#line 1743 "parse.c"
     break;
 
   case 37: /* serveropts1: MAX_REPOS_DISPLAY NUMBER  */
-#line 390 "parse.y"
+#line 402 "parse.y"
                                            {
-				new_srv->max_repos_display = (yyvsp[0].v.number);
+			if ((yyvsp[0].v.number) < 0) {
+				yyerror("max_repos_display is too small: %lld",
+				    (yyvsp[0].v.number));
+				YYERROR;
+			}
+			new_srv->max_repos_display = (yyvsp[0].v.number);
 		}
-#line 1739 "parse.c"
+#line 1756 "parse.c"
     break;
 
   case 38: /* serveropts1: MAX_COMMITS_DISPLAY NUMBER  */
-#line 393 "parse.y"
+#line 410 "parse.y"
                                              {
-			if ((yyvsp[0].v.number) > 0)
-				new_srv->max_commits_display = (yyvsp[0].v.number);
+			if ((yyvsp[0].v.number) <= 1) {
+				yyerror("max_commits_display is too small:"
+				    " %lld", (yyvsp[0].v.number));
+				YYERROR;
+			}
+			new_srv->max_commits_display = (yyvsp[0].v.number);
 		}
-#line 1748 "parse.c"
+#line 1769 "parse.c"
     break;
 
 
-#line 1752 "parse.c"
+#line 1773 "parse.c"
 
       default: break;
     }
@@ -1941,7 +1962,7 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 410 "parse.y"
+#line 431 "parse.y"
 
 
 struct keywords {
@@ -2537,59 +2558,8 @@ getservice(const char *n)
 	return (unsigned short)llval;
 }
 
-struct address *
-host_v4(const char *s)
-{
-	struct in_addr ina;
-	struct sockaddr_in *sain;
-	struct address *h;
-
-	memset(&ina, 0, sizeof(ina));
-	if (inet_pton(AF_INET, s, &ina) != 1)
-		return (NULL);
-
-	if ((h = calloc(1, sizeof(*h))) == NULL)
-		fatal(__func__);
-	sain = (struct sockaddr_in *)&h->ss;
-	got_sockaddr_inet_init(sain, &ina);
-	if (sain->sin_addr.s_addr == INADDR_ANY)
-		h->prefixlen = 0; /* 0.0.0.0 address */
-	else
-		h->prefixlen = -1; /* host address */
-	return (h);
-}
-
-struct address *
-host_v6(const char *s)
-{
-	struct addrinfo hints, *res;
-	struct sockaddr_in6 *sa_in6, *ra;
-	struct address *h = NULL;
-
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_INET6;
-	hints.ai_socktype = SOCK_DGRAM; /* dummy */
-	hints.ai_flags = AI_NUMERICHOST;
-	if (getaddrinfo(s, "0", &hints, &res) == 0) {
-		if ((h = calloc(1, sizeof(*h))) == NULL)
-			fatal(__func__);
-		sa_in6 = (struct sockaddr_in6 *)&h->ss;
-		ra = (struct sockaddr_in6 *)res->ai_addr;
-		got_sockaddr_inet6_init(sa_in6, &ra->sin6_addr,
-		    ra->sin6_scope_id);
-		if (memcmp(&sa_in6->sin6_addr, &in6addr_any,
-		    sizeof(sa_in6->sin6_addr)) == 0)
-			h->prefixlen = 0; /* any address */
-		else
-			h->prefixlen = -1; /* host address */
-		freeaddrinfo(res);
-	}
-
-	return (h);
-}
-
 int
-host_dns(const char *s, struct server *new_srv, int max,
+host(const char *s, struct server *new_srv, int max,
     in_port_t port, const char *ifname, int ipproto)
 {
 	struct addrinfo hints, *res0, *res;
@@ -2602,8 +2572,8 @@ host_dns(const char *s, struct server *new_srv, int max,
 		return (cnt);
 
 	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = PF_UNSPEC;
-	hints.ai_socktype = SOCK_DGRAM; /* DUMMY */
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM; /* DUMMY */
 	hints.ai_flags = AI_ADDRCONFIG;
 	error = getaddrinfo(s, NULL, &hints, &res0);
 	if (error == EAI_AGAIN || error == EAI_NODATA || error == EAI_NONAME)
@@ -2636,7 +2606,6 @@ host_dns(const char *s, struct server *new_srv, int max,
 		if (ipproto != -1)
 			h->ipproto = ipproto;
 		h->ss.ss_family = res->ai_family;
-		h->prefixlen = -1; /* host address */
 
 		if (res->ai_family == AF_INET) {
 			struct sockaddr_in *ra;
@@ -2703,7 +2672,6 @@ host_if(const char *s, struct server *new_srv, int max,
 		if (ipproto != -1)
 			h->ipproto = ipproto;
 		h->ss.ss_family = af;
-		h->prefixlen = -1; /* host address */
 
 		if (af == AF_INET) {
 			struct sockaddr_in *ra;
@@ -2736,41 +2704,6 @@ host_if(const char *s, struct server *new_srv, int max,
 	return (cnt);
 }
 
-int
-host(const char *s, struct server *new_srv, int max,
-    in_port_t port, const char *ifname, int ipproto)
-{
-	struct address *h;
-
-	h = host_v4(s);
-
-	/* IPv6 address? */
-	if (h == NULL)
-		h = host_v6(s);
-
-	if (h != NULL) {
-		if (port)
-			h->port = port;
-		if (ifname != NULL) {
-			if (strlcpy(h->ifname, ifname, sizeof(h->ifname)) >=
-			    sizeof(h->ifname)) {
-				log_warnx("%s: interface name truncated",
-				    __func__);
-				free(h);
-				return (-1);
-			}
-		}
-		if (ipproto != -1)
-			h->ipproto = ipproto;
-
-		if (add_addr(new_srv, h))
-			return -1;
-		return (1);
-	}
-
-	return (host_dns(s, new_srv, max, port, ifname, ipproto));
-}
-
 int
 is_if_in_group(const char *ifname, const char *groupname)
 {
